/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2005 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.updater.config;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;
import javax.xml.namespace.QName;

public class XemeliosUpdateModel implements XmlMarshallable {
	public static final transient String TAG = "xemelios-update";
        public static final transient QName QN = new QName(TAG);
	
	private ReleaseModel release = null;
	private TexteModel originalLocation = null;
	private UpdatePartModel xemeliosUpdates, configUpdates, featuresUpdates;
	private ForeignSitesModel foreignSites;

	public XemeliosUpdateModel(QName tag) {
		super();
	}

	public void addCharacterData(String cData) throws SAXException {
	}

	public void addChild(XmlMarshallable child, QName tagName) throws SAXException {
		if(ReleaseModel.QN.equals(tagName)) release = (ReleaseModel)child;
		else if(UpdateParser.ORIGINAL_LOCATION.equals(tagName)) originalLocation = (TexteModel)child;
		else if(UpdateParser.XEMELIOS_UPDATES.equals(tagName)) xemeliosUpdates = (UpdatePartModel)child;
		else if(UpdateParser.CONFIG_UPDATES.equals(tagName)) configUpdates = (UpdatePartModel)child;
		else if(UpdateParser.FEATURES_UPDATES.equals(tagName)) featuresUpdates = (UpdatePartModel)child;
		else if(ForeignSitesModel.QN.equals(tagName)) foreignSites = (ForeignSitesModel)child;
	}

	public XmlMarshallable getAttributes(XmlAttributes attributes) throws SAXException {
		return this;
	}

	public void marshall(XmlOutputter output) {
		output.startTag(TAG);
		if(release!=null) release.marshall(output);
		if(originalLocation!=null) originalLocation.marshall(output);
		if(xemeliosUpdates!=null) xemeliosUpdates.marshall(output);
		if(configUpdates!=null) configUpdates.marshall(output);
		if(featuresUpdates!=null) featuresUpdates.marshall(output);
		if(foreignSites!=null) foreignSites.marshall(output);
		output.endTag(TAG);
	}

	public void validate() throws InvalidXmlDefinition {
		if(release==null) throw new InvalidXmlDefinition("element release is mandatory for element "+TAG);
	}
	public XemeliosUpdateModel clone() {
		XemeliosUpdateModel xu = new XemeliosUpdateModel(QN);
		xu.release=release.clone();
		if(originalLocation!=null) xu.originalLocation=originalLocation.clone();
		if(xemeliosUpdates!=null) xu.xemeliosUpdates=xemeliosUpdates.clone();
		if(configUpdates!=null) xu.configUpdates=configUpdates.clone();
		if(featuresUpdates!=null) xu.featuresUpdates=featuresUpdates.clone();
		if(foreignSites!=null) xu.foreignSites=foreignSites.clone();
		return xu;
	}

	public UpdatePartModel getConfigUpdates() {
		return configUpdates;
	}

	public void setConfigUpdates(UpdatePartModel configUpdates) {
		this.configUpdates = configUpdates;
	}

	public UpdatePartModel getFeaturesUpdates() {
		return featuresUpdates;
	}

	public void setFeaturesUpdates(UpdatePartModel featuresUpdates) {
		this.featuresUpdates = featuresUpdates;
	}

	public ForeignSitesModel getForeignSites() {
		return foreignSites;
	}

	public void setForeignSites(ForeignSitesModel foreignSites) {
		this.foreignSites = foreignSites;
	}

	public TexteModel getOriginalLocation() {
		return originalLocation;
	}

	public void setOriginalLocation(TexteModel originalLocation) {
		this.originalLocation = originalLocation;
	}

	public ReleaseModel getRelease() {
		return release;
	}

	public void setRelease(ReleaseModel release) {
		this.release = release;
	}

	public UpdatePartModel getXemeliosUpdates() {
		return xemeliosUpdates;
	}

	public void setXemeliosUpdates(UpdatePartModel xemeliosUpdates) {
		this.xemeliosUpdates = xemeliosUpdates;
	}

    public XmlMarshallable getChildToModify(String uri, String localName, String qName, Attributes atts) {
        return null;
    }

    public QName getQName() {
        return QN;
    }
}
